from django.template.loader import get_template


def test_home():
    page = get_template('page.html')
    assert page is not None


def test_group_about():
    page = get_template('(test_group)/about/page.html')
    assert page is not None


def test_group_info():
    page = get_template('(test_group)/info/page.html')
    assert page is not None


def test_nested_lv1():
    page = get_template('test_nested/page.html')
    assert page is not None


def test_nested_lv2():
    page = get_template('test_nested/test_nested_2/page.html')
    assert page is not None


def test_slug():
    page = get_template('test_slug/[slug]/page.html')
    assert page is not None



"""
MIT License

Copyright (c) 2024-present STACiA

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import logging
from pathlib import Path
from typing import Any, Callable, List, get_type_hints

from django.urls.resolvers import RoutePattern, URLPattern

from . import utils

# fmt: off
__all__ = (
    'init',
)
# fmt: on


log = logging.getLogger('django_app_router')


def _normalize_route(
    route_path: Path,
    func: Callable[..., Any],
    *,
    trailing_slash: bool = True,
) -> str:

    # TODO: route_path regex validation

    normal_route = []
    func_type_hints = get_type_hints(func)

    route_path_string = str(route_path)
    for param in route_path_string.split('/'):
        if param.startswith('[') and param.endswith(']'):
            param = param[1:-1]
            param_type = func_type_hints.get(param, str)
            normal_route.append(f'<{param_type.__name__}:{param}>')
        elif param == '.':
            normal_route.append('')
        elif param.startswith('(') and param.endswith(')'):
            continue
        else:
            normal_route.append(param)

    route = '/'.join(normal_route)
    if trailing_slash and route != '' and not route.endswith('/'):
        route += '/'
    return route


def init(template_path: Path) -> List[URLPattern]:

    url_patterns = []

    if not template_path.exists():
        raise FileNotFoundError(f'{template_path} does not exist')

    pages = template_path.glob('**/page.py')

    for page_path in pages:

        module = utils.get_module_from_path(page_path)

        if not hasattr(module, 'page'):
            log.warning(f'Should have a page function in {page_path}')
            continue

        func = getattr(module, 'page')

        route = page_path.relative_to(template_path).parent

        route_string = _normalize_route(route, func)
        route_name = func.__doc__

        # TODO: support kwargs
        kwargs: Any = None

        route_pattern = RoutePattern(route_string, name=route_name, is_endpoint=True)
        url_pattern = URLPattern(route_pattern, func, kwargs, route_name)

        url_patterns.append(url_pattern)

    return url_patterns



"""
Django App Router
~~~~~~~~~~~~~~~~~~~
A simple Django app to route requests, Inspired by Next.js App Router.
:copyright: (c) 2024-present STACiA
:license: MIT, see LICENSE for more details.
"""

__title__ = 'django_app_router'
__author__ = 'STACiA'
__license__ = 'MIT'
__copyright__ = 'Copyright 2024-present STACiA(staciax)'
__version__ = '0.0.4'


from typing import Literal, NamedTuple

from .app_router import *


class VersionInfo(NamedTuple):
    major: int
    minor: int
    micro: int
    release: Literal['alpha', 'beta', 'final']


version_info: VersionInfo = VersionInfo(major=0, minor=0, micro=4, release='final')

del NamedTuple, Literal, VersionInfo



import pytest
from django.http.response import HttpResponse
from django.test.client import Client


@pytest.mark.urls('tests.urls')
def test_home(client: Client):
    response = client.get('/')
    assert response is not None
    assert isinstance(response, HttpResponse)
    assert response.status_code == 200

    assert b'<h1>Home</h1>' in response.content


@pytest.mark.urls('tests.urls')
def test_group_about(client: Client):
    response = client.get('/about/')
    assert response is not None
    assert isinstance(response, HttpResponse)
    assert response.status_code == 200

    assert b'<h1>About</h1>' in response.content


@pytest.mark.urls('tests.urls')
def test_group_info(client: Client):
    response = client.get('/info/')
    assert response is not None

    assert isinstance(response, HttpResponse)
    assert response.status_code == 200

    assert b'<h1>Info</h1>' in response.content


@pytest.mark.urls('tests.urls')
def test_nested_lv1(client: Client):
    response = client.get('/test_nested/')
    assert isinstance(response, HttpResponse)
    assert response.status_code == 200

    assert b'<h1>Nested</h1>' in response.content


@pytest.mark.urls('tests.urls')
def test_nested_lv2(client: Client):
    response = client.get('/test_nested/test_nested_2/')
    assert isinstance(response, HttpResponse)
    assert response.status_code == 200

    assert b'<h1>Nested2</h1>' in response.content


@pytest.mark.urls('tests.urls')
def test_slug(client: Client):
    slug = 'test-slug'
    response = client.get('/test_slug/{slug}/'.format(slug=slug))
    assert isinstance(response, HttpResponse)
    assert response.status_code == 200

    assert (f'Slug: {slug}').encode() in response.content
